/**
 * 使用：
 * import Middleware from 's-middleware'
 *
 * var httpMiddleware = new Middleware<Request, Response>();
 *
 * httpMiddleware
 *  .use((request: Request, next: NextType): Response => {
 *      //doing something
 *  })
 *  ...
 *  .endWith(async (request: Request): Response => {
 *      return await axios.Request(request);
 *  })
 *
 */
export declare type nextType<T, K> = (opts: T) => K;
export declare type middlewareType<T, K> = (opts: T, next?: nextType<T, K>) => K;
export declare type endMiddleware<T, K> = (opts: T) => K;
export default class Middleware<T, K> {
    private middlewareQueue;
    private end;
    private start;
    /**
     * 注册中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     *
     * eg: new Middleware<string, string>().use(middleware1, middleware2);
     */
    use(...middlewareList: Array<middlewareType<T, K>>): this;
    /**
     * 移除中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     *
     * eg: new Middleware<string, string>().unuse(middleware1, middleware2);
     */
    unuse(middleware: middlewareType<T, K>): boolean;
    /**
     * 运行中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     *
     * eg: new Middleware<string, string>().run(1)
     */
    run(opts: T, index?: number): K | undefined;
    /**
     * 第一个中间件，始终被第一个调用
     * @param { middlewareType } middlewareList 中间件列表
     * @returns this
     *
     * eg: new Middleware<string, string>().startWith(xxxMiddleware)
     */
    startWith(start: middlewareType<T, K>): this;
    /**
     * 最后一个中间件，始终最后被调用
     * @param { middlewareType } middlewareList 中间件列表
     * @returns this
     *
     * eg: new Middleware<string, string>().startWith(xxxMiddleware)
     */
    endWith(end: endMiddleware<T, K>): this;
    private runMiddleware;
    private isInMiddleware;
    private isFunction;
    private toType;
}
