/**
 * 使用：
 * import Middleware from 's-middleware'
 * 
 * var httpMiddleware = new Middleware<Request, Response>();
 * 
 * httpMiddleware
 *  .use((request: Request, next: NextType): Response => {
 *      //doing something
 *  })
 *  ...
 *  .endWith(async (request: Request): Response => {
 *      return await axios.Request(request);
 *  })
 * 
 */

export type nextType<T, K> = (opts: T) => K;
export type middlewareType<T, K> = (opts: T, next?: nextType<T, K>) => K;
export type endMiddleware<T, K> = (opts: T) => K;

export default class Middleware<T, K> {
    private middlewareQueue: Array<middlewareType<T, K>> = [];
    private end: endMiddleware<T, K> = (opts) => this.toType(opts);
    private start: middlewareType<T, K> = (opts, next) => next(opts);

    /**
     * 注册中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     * 
     * eg: new Middleware<string, string>().use(middleware1, middleware2);
     */
    public use(...middlewareList: Array<middlewareType<T, K>>): this {
        middlewareList.forEach((middleware: middlewareType<T, K>) => {
            if (this.isFunction(middleware) && !this.isInMiddleware(middleware)) {
                this.middlewareQueue.push(middleware);
            }
        });
        return this;
    }

    /**
     * 移除中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     * 
     * eg: new Middleware<string, string>().unuse(middleware1, middleware2);
     */
    public unuse(middleware: middlewareType<T, K>): boolean {
        const index = this.middlewareQueue.indexOf(middleware);

        if (index > -1) {
            this.middlewareQueue.splice(index, 1);
            return !this.isInMiddleware<T, K>(middleware);
        }

        return true;
    }

    /**
     * 运行中间件
     * @param { middlewareType[] } middlewareList 中间件列表
     * @returns this
     * 
     * eg: new Middleware<string, string>().run(1)
     */
    public run(opts: T, index = 0): K | undefined {
        return this.start(opts, (nextOpts) => {
            return this.runMiddleware(nextOpts, index);
        });
    }

    /**
     * 第一个中间件，始终被第一个调用
     * @param { middlewareType } middlewareList 中间件列表
     * @returns this
     * 
     * eg: new Middleware<string, string>().startWith(xxxMiddleware)
     */
    public startWith(start: middlewareType<T, K>): this {
        this.start = start;
        return this;
    }

    /**
     * 最后一个中间件，始终最后被调用
     * @param { middlewareType } middlewareList 中间件列表
     * @returns this
     * 
     * eg: new Middleware<string, string>().startWith(xxxMiddleware)
     */
    public endWith(end: endMiddleware<T, K>): this {
        this.end = end;
        return this;
    }

    private runMiddleware(opts: T, index = 0): K | undefined{
        const toNextMiddleware = (nextOpts: T) => this.runMiddleware(nextOpts, index + 1);

        if (this.isFunction(this.middlewareQueue[index])) {
            return this.middlewareQueue[index](opts, toNextMiddleware);
        } else {
            return this.end(opts);
        }
    }

    private isInMiddleware<T, K>(middleware: middlewareType<T, K>, middlewareQueue: Array<middlewareType<T, K>> = []): boolean {
        return !middlewareQueue.every((item) => item !== middleware);
    }

    private isFunction(fn: Function): boolean {
        return typeof fn === 'function';
    }

    private toType<K>(obj: Object): K {
        return <K>obj;
    }
}